#include "cprotocole.h"

using namespace std;

CProtocole::CProtocole(QObject *parent, QString nomPort)
{
    mEtat = AVANT_MISSION;
    mEtatData = RECH_DEBUT_TRAME;
    mSp = new CSerialPort(this, nomPort);
    mSp->initialiser(QSerialPort::Baud9600,
                     QSerialPort::Data8,
                     QSerialPort::NoParity,
                     QSerialPort::OneStop,
                     QSerialPort::NoFlowControl);
    mSp->ouvrirPort();
    connect(mSp, SIGNAL(readyRead(QByteArray)), this, SLOT(onReadyRead(QByteArray)));
}

CProtocole::~CProtocole()
{
    delete mSp;
}

US CProtocole::retrouveCrc(QByteArray baCrc)
{
    return (unsigned short) baCrc.toInt(NULL, 16);
}

US CProtocole::crc(UC *tab, int nb)
{
    UC nbDec,
       yaUn,
       ind;
    US crc;

    crc = 0xFFFF;
    ind = 0;

    do {
        crc ^= (US)tab[ind];
        nbDec = 0;
        do {
            if ((crc & 0x0001) == 1)
                yaUn = 1;
            else
                yaUn = 0;
            crc >>= 1;
            if (yaUn)
                crc ^= 0xA001;
            nbDec++;
        } while (nbDec < 8);
        ind++;
    } while (ind < nb);
    return(crc);
}


US CProtocole::calculCrc(QByteArray qbaTrame, int lgTrame)
{
    int lgCorps = lgTrame - 9;
    int ok = 0;
    int noOk = -1;
    this->crcRecu = retrouveCrc(qbaTrame.mid(lgTrame-4,4));
    this->crcCalc = crc((unsigned char*)qbaTrame.mid(4,lgCorps-4).toStdString().c_str(), lgCorps);
    if(crcRecu == crcCalc)
    {
        return ok;
    }
    else
    {
        return noOk;
    }
}

QString CProtocole::decodeTrame(QByteArray trame)
{
    int fonc;
    QByteArray fonc2;


    fonc2 = trame.mid(2,2);
    fonc = fonc2.toInt(NULL,16);

    switch(fonc)
    {
    case 0x00:
        emit sigReponse("DEBUT MISSION");
        break;
    case 0x03:
        emit sigReponse("ENVOI DONNEES CONFIG");
        break;
    case 0x04:
        emit sigReponse("ENVOI PARAM CAPTEURS");
        break;
    case 0x09:
        emit sigReponse("MISE A HEURE EDD");
        break;
    case 0xAA:
        emit sigReponse("ACK");
        break;
    case 0xFF:
        emit sigReponse("TEST COM EDD");
        qDebug() << "CProtocole::signalTest";
        break;
    case 0x11:
        emit sigReponse("DEPART ACQUISITION");
        break;
    case 0x12:
        emit sigReponse("STOP ACQUISITION");
        break;
    case 0x15:
        emit sigReponse("ORDRE CAMERA");
        break;
    case 0x16:
        emit sigReponse("MODIF PARAMS INCRUSTATION");
        break;
    case 0x17:
        emit sigReponse("ENVOI LOT DONNEES INSTANTANEE");
        break;
    case 0x18:
        emit sigReponse("MODIF INTERVALLE");
        break;
    case 0x19:
        emit sigReponse("STOP MISSION");
        break;
    default:
        break;
    }
}

///////////////////////// AVANT MISSION /////////////////////////

int CProtocole::debutMission()
{
    QByteArray qbaTrame;
    if(mEtat != PENDANT_MISSION)
    {
        mEtat = PENDANT_MISSION;
        qbaTrame = "\x02[00]\x03";
        mSp->ecrire(qbaTrame,6);
    }
    return 0;
}

bool CProtocole::envoiDonneesConfig(QByteArray qbaTrame)
{
    US crc = calculCrc(qbaTrame, 11);
    if(mEtat != AVANT_MISSION)
    {
        return -1;
    }
    qbaTrame = "\x02[03]\x03";
    qbaTrame[7] = (unsigned char)crc >> 8;
    qbaTrame[8] = (unsigned char)crc;
    mSp->ecrire(qbaTrame, 15);

    emit afficherTexte(QString(qbaTrame));

    QFile file(mConfData.nomMission);
    if(!file.open(QFile::ReadWrite))
    {
        qDebug() << "Erreur création fichier mission !";
        return -1;
    }

    QTextStream out (&file);
    out << qbaTrame;
    file.close();
    qDebug() << "Fichier cree !";

    if(qbaTrame.at(4+4) == '1')
    {
        mConfData.emMesVersGcs = true;
    }
    else
    {
        mConfData.emMesVersGcs = false;
    }
    return mConfData.emMesVersGcs;
}

int CProtocole::envoiParamsCapteur(QByteArray qbaTrame)
{
    US crc = calculCrc(qbaTrame, 11);
    if(mEtat != AVANT_MISSION)
    {
        return -1;
    }
    qbaTrame = "\x02[04]\x03";
    qbaTrame[7] = (unsigned char)crc >> 8;
    qbaTrame[8] = (unsigned char)crc;
    mSp->ecrire(qbaTrame, 15);

    emit afficherTexte(QString(qbaTrame));

    return 0;
}

int CProtocole::miseAHeureEDD(QByteArray qbaTrame)
{
    US crc = calculCrc(qbaTrame, 11);
    if(mEtat != AVANT_MISSION)
    {
        return -1;
    }
    qbaTrame = "\x02[09]\x03";
    qbaTrame[7] = (unsigned char)crc >> 8;
    qbaTrame[8] = (unsigned char)crc;
    mSp->ecrire(qbaTrame, 15);

    emit afficherTexte(QString(qbaTrame));

    QList<QByteArray> dt;
    QProcess proc;
    QString command = "sudo date";
    QStringList arg;

    dt = qbaTrame.mid(4,5).split(';');

    QString theDate = QString(dt.at(0))+QString(dt.at(1))+QString(dt.at(2));
    QString theTime = QString(dt.at(3))+QString(dt.at(4))+QString(dt.at(5));

    arg << theDate + theTime;
    proc.start(command, arg);
    sleep(2);
    return 0;
}

int CProtocole::testComm()
{
    QByteArray qbaTrame;
    if(mEtat != AVANT_MISSION)
    {
        return -1;
    }
    qbaTrame = "\x02[FF]\x03";
    mSp->ecrire(qbaTrame, 6);

    return 0;
}

/////////////////////// PENDANT MISSION /////////////////////////

int CProtocole::debutAcquisition(QByteArray qbaTrame)
{

}

int CProtocole::stopAcquisition(QByteArray qbaTrame)
{

}

int CProtocole::camera(QByteArray qbaTrame)
{

}

int CProtocole::modifParamIncruste(QByteArray qbaTrame)
{

}

int CProtocole::mesuresInstantane(QByteArray qbaTrame)
{

}

void CProtocole::onReadyRead(QByteArray ba)
{
    for(int i=0; i<ba.size();i++)
    {
        switch (mEtatData)
        {
            case RECH_DEBUT_TRAME:

                if (ba.at(i) == STX)
                {
                    mEtatData = SAUVE_TRAME;
                }
            break;
            case SAUVE_TRAME:

                if(ba.at(i) == ETX)
                {
                    mEtatData = RECH_DEBUT_TRAME;
                    decodeTrame(mTrame);
                }
        }
    }
}




        /*if (car.at(0) == ETX) {  // fin de trame
            qDebug() << "CCommuniquer::onReadyRead: trame finie = " << mTrame;
            int res=protocole(mTrame, mTrame.size());
            //envoi de l'acquittement si tout va bien
            if (res>0) {
                mPs->write("\x02[AA]\x03");
                qDebug() << "CCommuniquer::onReadyRead: ACK envoyé";
            } // if res
            mEtatData = RECH_DEBUT_TRAME;
        } // if CARFIN
        else {
            mTrame.append(car);
//                qDebug() << "CCommuniquer::onReadyRead: trame en cours = " << mTrame;
        } // else
        break;
    default:
        qDebug() << "CCommuniquer::onReadyRead: Pb état réception.";
    } // sw
}*/
