#ifndef protocole_hpp
#define protocole_hpp

#include <stdio.h>
#include <QTimer>
#include <QFile>
#include <QDebug>
#include <QProcess>
#include <unistd.h>
#include "global.h"
#include "cserialport.h"

#define STX 0x02
#define ETX 0x03

typedef unsigned char UC;
typedef unsigned short US;

typedef enum {
    AVANT_MISSION = 0,
    PENDANT_MISSION,
    APRES_MISSION
} T_ETAT_SYSTEM;

typedef enum {
    RECH_DEBUT_TRAME = 0,
    SAUVE_TRAME
} T_ETAT_COMM_DATA;

typedef struct {
    char nomMission[50];
    bool emMesVersGcs;
} T_DATACONFIG;

class CProtocole : public QObject
{
    Q_OBJECT
public:
    explicit CProtocole(QObject *parent, QString nomPort = "/dev/ttyUSB0");
    ~CProtocole();
////////////////////  AVANT MISSION  ////////////////////
    int debutMission();
    bool envoiDonneesConfig(QByteArray qbaTrame);
    int envoiParamsCapteur(QByteArray qbaTrame);
    int miseAHeureEDD(QByteArray qbaTrame);
    int testComm();

///////////////////  PENDANT MISSION ////////////////////
    int debutAcquisition(QByteArray qbaTrame);
    int stopAcquisition(QByteArray qbaTrame);
    int camera(QByteArray qbaTrame);
    int modifParamIncruste(QByteArray qbaTrame);
    int mesuresInstantane(QByteArray qbaTrame);

private:

    T_DATACONFIG mConfData;
    T_ETAT_SYSTEM mEtat;
    T_ETAT_COMM_DATA mEtatData;
    US crc(UC *tab, int nb);
    US retrouveCrc(QByteArray baCrc);
    US crcRecu;
    US crcCalc;
    QTimer *mTimer;
    QByteArray mTrame;
    T_Mes *mData;
    QObject *mProto;
    CSerialPort *mSp;
    int ACK(QByteArray qbaTrame, int lgTrame);
    US calculCrc(QByteArray qbaTrame, int lgTrame);
    QString decodeTrame(QByteArray trame);

signals:
    void afficherTexte(QString txt);
    void sigReponse(QString rep);
private slots:
    void onReadyRead(QByteArray ba);
};
#endif
