#ifndef GLOBAL_H
#define GLOBAL_H

#include <QSharedMemory>

#define NOMFIC "./ditRasp"  // pour générer la clef de la file de messages
#define LETTRE 'd'    // pour générer la clef de la file de messages
#define KEY "./ditRasp"     // clef pour le segment de mémoire partagé
#define AFFMAX 50      // long max de la valeur de la mesure
#define NBMAXCAPT 9     // Nb max de capteurs sur le drone (9 affichage au max d'incrustation)
#define TYPE_MESS_MESURE 1  // type du message pour transmettre une mesure
#define TYPE_MESS_INCRUSTER 2 //      ""    "       changer l'incrustation
#define TYPE_MESS_TIMERINC 21     //    ""              changer l'interval du timer d'incrustation
#define TYPE_MESS_ORDRE_CAMERA 5  // pour prendre une photo, start/stop REC vidéo
#define TYPE_MESS_ACK_ORDRE 11  // pour communiquer ACK des ordres
#define TYPE_MESS_STOP_THREAD  // pour arrêt des threads capteurs

typedef unsigned char UC;
typedef unsigned short US;

// structure du fichier de configuration config.ini
// Ce fichier contient toutes les mesures à incruster.
typedef struct {
    int noMes;                  // numéro de la mesure
    int adrCapteur;             // Adresse ou fichier d'accès au capteur en hexa sans particule
    int posL;                   // N° ligne affichage incrustation
    int posC;                   // N° colonne affichage incrustation
    char nomClasse[AFFMAX];     // nom de la classe de gestion du capteur
    char nomMes[AFFMAX];        // nom de la mesure
    char symbUnit[AFFMAX];      // Symbole de l'unité de la mesure
    char textUnit[AFFMAX];      // Texte de l'unité
    char valMes[AFFMAX];        // Valeur instantannée de la mesure
} T_Mes;

// type de message pour une mesure
typedef union umess {
    double valDouble;
    int valInt;
    char valCh[AFFMAX];
} U_Mess;

typedef struct {
    long type;
    int noMesure;
    U_Mess corps;
} T_MessMes;

// type de message pour autoriser les mesures à incruster
// -1 si pas d'incrustation de la mesure
typedef struct {
    long type;
    int c[9];  // nO du capteur à incruster
} T_MessInc;

// type pour un ordre
typedef struct {
    long type;
    char ordre[255];
} T_MessOrdre;

// type pour changer l'interval d'un timer
typedef struct {
    long type;
    int interval;
    bool enable; // pour start ou stop timer
} T_MessIntTimer;

#endif // GLOBAL_H


