// Programme incrustateur vidéo MAX7456 pour Raspberry Pi avec librairie BCM2835

// g++ -o incrustateur4 bcm2835.cpp incrustateur4_2018_01_16.cpp 

#include <iostream>
#include <bcm2835.h>
#include <stdio.h>

using namespace std;

//using namespace std;

// Declaration GPIO pour CS
#define CS RPI_BPLUS_GPIO_J8_22 // Numéro de la broche sur le GPIO

//MAX7456 opcodes
#define DMM_reg   0x04
#define DMAH_reg  0x05
#define DMAL_reg  0x06
#define DMDI_reg  0x07
#define VM0_reg   0x00
#define VM1_reg   0x01

#define DMDO_reg  0x07

// video mode register 0 bits
#define VIDEO_BUFFER_DISABLE 0x01
#define MAX7456_RESET 0x02
#define VERTICAL_SYNC_NEXT_VSYNC 0x04
#define OSD_ENABLE 0x08
#define SYNC_MODE_AUTO 0x00
#define SYNC_MODE_INTERNAL 0x30
#define SYNC_MODE_EXTERNAL 0x20
#define VIDEO_MODE_PAL 0x40
#define VIDEO_MODE_NTSC 0x00

// video mode register 1 bits
// duty cycle is on_off
#define BLINK_DUTY_CYCLE_50_50 0x00
#define BLINK_DUTY_CYCLE_33_66 0x01
#define BLINK_DUTY_CYCLE_25_75 0x02
#define BLINK_DUTY_CYCLE_75_25 0x03

// blinking time
#define BLINK_TIME_0 0x00
#define BLINK_TIME_1 0x04
#define BLINK_TIME_2 0x08
#define BLINK_TIME_3 0x0C

// background mode brightness (percent)
#define BACKGROUND_BRIGHTNESS_0 0x00
#define BACKGROUND_BRIGHTNESS_7 0x01
#define BACKGROUND_BRIGHTNESS_14 0x02
#define BACKGROUND_BRIGHTNESS_21 0x03
#define BACKGROUND_BRIGHTNESS_28 0x04
#define BACKGROUND_BRIGHTNESS_35 0x05
#define BACKGROUND_BRIGHTNESS_42 0x06
#define BACKGROUND_BRIGHTNESS_49 0x07

#define BACKGROUND_MODE_GRAY 0x40

//MAX7456 commands
#define CLEAR_display 0x04
#define CLEAR_display_vert 0x06
#define END_string 0xff

// with PAL
// all VM0_reg commands need bit 6 set
#define ENABLE_display 0x48
#define ENABLE_display_vert 0x4c
#define MAX7456_reset 0x42
#define DISABLE_display 0x40


#define WHITE_level_80 0x03
#define WHITE_level_90 0x02
#define WHITE_level_100 0x01
#define WHITE_level_120 0x00

// with PAL
#define MAX_screen_size 480
#define MAX_screen_rows 16

//////////////////////////////////////////////////////////////
// Transposition de la table ASCII (non standard) integrée à l'EEPROM du MAX7456
unsigned char convert_ascii (int character)
{
 unsigned char lookup_char;

  if (character == 32)
    lookup_char = 0x00; // blank space
  else if (character == 48)
    lookup_char = 0x0a; // 0
  else if ((character > 48) && (character < 58))
    lookup_char = (character - 48); // 1-9
  else if ((character > 64) && (character < 91))
    lookup_char = (character - 54); // A-Z
  else if ((character > 96) && (character < 123))
    lookup_char = (character - 60); // a-z
  else if (character == 34)
    lookup_char = 0x48; // "
  else if (character == 39)
    lookup_char = 0x46; // '
  else if (character == 40)
    lookup_char = 0x3f; // (
  else if (character == 41)
    lookup_char = 0x40; // )
  else if (character == 44)
    lookup_char = 0x45; // ,
  else if (character == 45)
    lookup_char = 0x49; // -
  else if (character == 46)
    lookup_char = 0x41; // .
  else if (character == 47)
    lookup_char = 0x47; // /
  else if (character == 58)
    lookup_char = 0x44; // :
  else if (character == 59)
    lookup_char = 0x43; // ;
  else if (character == 60)
    lookup_char = 0x4a; // <
  else if (character == 62)
    lookup_char = 0x4b; // >
  else if (character == 63)
    lookup_char = 0x42; // ?
  else if (character == 64)
    lookup_char = 0x4c; // @
  else
    lookup_char = 0x00; // out of range, blank space

 return (lookup_char);

}

/////////////////////////////////////////////////////////////////////////////////////////////////////

// =================== ECRITURE SUR L'ECRAN
void OSD_write_to_screen(const char s[], unsigned char x, unsigned char y, unsigned char blink, unsigned char invert)
{
  unsigned int linepos;
  unsigned char local_count;
  unsigned char settings, char_address_hi, char_address_lo;
  unsigned char screen_char;
  
  local_count = 0;
  
  char_address_hi = 0;
  char_address_lo = 0;
  
char spi_Out[40];
char spi_In[40];
int cursor;

  // convert x,y to line position
  linepos = y*30+x;

  // divide in to hi & lo byte
  char_address_hi = linepos >> 8;
  char_address_lo = linepos;

  // clear the screen
  //OSD_clear();

  settings = 00000001;

  // set blink bit
  if (blink) {
    settings |= (1 << 4);	 // forces nth bit of x to be 1.  all other bits left alone.
    //x &= ~(1 << n);	// forces nth bit of x to be 0.  all other bits left alone.
  }
  // set invert bit
  if (invert){
    settings |= (1 << 3);	 // forces nth bit of x to be 1.  all other bits left alone.
  }

cursor=0;
       //bcm2835_gpio_write(CS, LOW);
  spi_Out[cursor]=DMM_reg;
cursor++; 
  spi_Out[cursor]=settings;
cursor++; 
//  bcm2835_spi_transfer(DMM_reg); //dmm
//  bcm2835_spi_transfer(settings);

  spi_Out[cursor]=DMAH_reg; 
cursor++; 
  spi_Out[cursor]=char_address_hi;
cursor++; 

 // bcm2835_spi_transfer(DMAH_reg); // set start address high
 // bcm2835_spi_transfer(char_address_hi);


  spi_Out[cursor]=DMAL_reg; 
cursor++; 
  spi_Out[cursor]=char_address_lo;
cursor++; 


 // bcm2835_spi_transfer(DMAL_reg); // set start address low
 // bcm2835_spi_transfer(char_address_lo);

  while(s[local_count]!='\0') // write out full screen
  {
    screen_char = s[local_count];
    screen_char= convert_ascii(screen_char);
  spi_Out[cursor]=DMDI_reg;
cursor++;  
  spi_Out[cursor]=screen_char;
cursor++; 

//    bcm2835_spi_transfer(DMDI_reg);
//    bcm2835_spi_transfer(screen_char);
    local_count++;
  }
  
  spi_Out[cursor]=DMDI_reg;
cursor++;  
  spi_Out[cursor]=END_string;
cursor++; 

//  bcm2835_spi_transfer(DMDI_reg);
//  bcm2835_spi_transfer(END_string);

  spi_Out[cursor]=VM0_reg;
cursor++;  
  spi_Out[cursor]=ENABLE_display_vert;
cursor++; 
	bcm2835_spi_transfernb(spi_Out, spi_In, cursor);

//  bcm2835_spi_transfer(VM0_reg); // turn on screen next vertical. En transparence sur écran
//  bcm2835_spi_transfer(ENABLE_display_vert);
       //bcm2835_gpio_write(CS, HIGH);
  }
//////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////
void Max7456 ()
{
bcm2835_spi_transfer(VM0_reg);
bcm2835_spi_transfer(VERTICAL_SYNC_NEXT_VSYNC|OSD_ENABLE|SYNC_MODE_AUTO);
}
///////////////////////////////////////////////////////////////

 /////////////////////////////////////////////////////////////
void incrustationPhrase()
{
   char phrase1[] =  "BTS SN 2016-2017 z"; 
   char phrase2[] =  "Projet Drone    ";          

      	OSD_write_to_screen(phrase1, 1, 3, 0,0);  
	bcm2835_delay(1000);
      	OSD_write_to_screen(phrase2, 1, 7, 0,0);  
	bcm2835_delay(1000);	
   }  
////////////////////////////////////////////////////////////////////////////////////////////////

int main(int argc, char **argv)
{
int x;

char spiOut[34];
char spiIn[34];
int curseur;


   if (!bcm2835_init())
    {
      printf("bcm2835_init failed. Are you running as root??\n");
      return 1;
    }
    if (!bcm2835_spi_begin())
    {
      printf("bcm2835_spi_begin failedg. Are you running as root??\n");
      return 1;
    }
    bcm2835_spi_setBitOrder(BCM2835_SPI_BIT_ORDER_MSBFIRST);      // The default
    bcm2835_spi_setDataMode(BCM2835_SPI_MODE0);                   // The default
    bcm2835_spi_setClockDivider(BCM2835_SPI_CLOCK_DIVIDER_64); // The default  65536
    bcm2835_spi_chipSelect(BCM2835_SPI_CS0);                      // The default
    bcm2835_spi_setChipSelectPolarity(BCM2835_SPI_CS0, LOW);      // the default
    
    	bcm2835_gpio_fsel(CS, BCM2835_GPIO_FSEL_OUTP);
        bcm2835_gpio_write(CS, HIGH);

cout<<"Hello !!"<<endl;

// force soft reset sur Max7456
       //bcm2835_gpio_write(CS, LOW);
spiOut[0]=VM0_reg; 
spiOut[1]=MAX7456_reset;
	bcm2835_spi_transfernb(spiOut, spiIn, 2);
//bcm2835_spi_transfer(VM0_reg);
//bcm2835_spi_transfer(MAX7456_reset);

       //bcm2835_gpio_write(CS, HIGH);
bcm2835_delay(500);

// set all rows to same charactor white level, 90%
//  digitalWrite(MAX7456SELECT,LOW);
       //bcm2835_gpio_write(CS, LOW);
curseur=0;
  for (x = 0; x < MAX_screen_rows; x++)
  {
spiOut[curseur]=x + 0x10; 
curseur++;
spiOut[curseur]=WHITE_level_90;
curseur++;
//    bcm2835_spi_transfer(x + 0x10);
//    bcm2835_spi_transfer(WHITE_level_90);
  }


  // Mise en service du Max7456
spiOut[curseur]=VM0_reg;
curseur++; 
spiOut[curseur]=VERTICAL_SYNC_NEXT_VSYNC|OSD_ENABLE;
//	bcm2835_spi_transfernb(spiOut, spiIn, 2);
curseur++;
	bcm2835_spi_transfernb(spiOut, spiIn, curseur);

  //bcm2835_spi_transfer(VM0_reg);
  //bcm2835_spi_transfer(VERTICAL_SYNC_NEXT_VSYNC|OSD_ENABLE); 
       //bcm2835_gpio_write(CS, HIGH);
  bcm2835_delay(100);

  incrustationPhrase();

    	bcm2835_spi_end();
    	bcm2835_close();
	cout<<"Terminé !!"<<endl;
    	return 0;
}

